@@ -6,13 +6,9 @@ from nassl import _nassl
 from nassl._nassl import WantReadError, OpenSSLError, WantX509LookupError
 
 from enum import IntEnum
-from typing import List, Any
+from typing import List, Any, Tuple
 
-try:
-    from typing import Protocol
-except ImportError:
-    # Will happen on Python 3.7
-    from typing_extensions import Protocol  # type: ignore
+from typing import Protocol
 
 
 from typing import Optional
@@ -36,6 +32,17 @@ class OpenSslVerifyEnum(IntEnum):
     CLIENT_ONCE = 4
 
 
+class OpenSslDigestNidEnum(IntEnum):
+    """SSL digest algorithms used for the signature algorithm, per obj_mac.h."""
+
+    MD5 = 4
+    SHA1 = 64
+    SHA224 = 675
+    SHA256 = 672
+    SHA384 = 673
+    SHA512 = 674
+
+
 class OpenSslVersionEnum(IntEnum):
     """SSL version constants."""
 
@@ -123,7 +130,9 @@ class BaseSslClient(ABC):
             self._ssl.set_tlsext_host_name(server_name_indication)
 
     def _init_base_objects(
-        self, ssl_version: OpenSslVersionEnum, underlying_socket: Optional[socket.socket]
+        self,
+        ssl_version: OpenSslVersionEnum,
+        underlying_socket: Optional[socket.socket],
     ) -> None:
         """Setup the socket and SSL_CTX objects."""
         self._is_handshake_completed = False
@@ -133,9 +142,7 @@ class BaseSslClient(ABC):
         # A Python socket handles transmission of the data
         self._sock = underlying_socket
 
-    def _init_server_authentication(
-        self, ssl_verify: OpenSslVerifyEnum, ssl_verify_locations: Optional[Path]
-    ) -> None:
+    def _init_server_authentication(self, ssl_verify: OpenSslVerifyEnum, ssl_verify_locations: Optional[Path]) -> None:
         """Setup the certificate validation logic for authenticating the server."""
         self._ssl_ctx.set_verify(ssl_verify.value)
         if ssl_verify_locations:
@@ -154,13 +161,16 @@ class BaseSslClient(ABC):
     ) -> None:
         """Setup client authentication using the supplied certificate and key."""
         if client_certificate_chain is not None and client_key is not None:
-            self._use_private_key(client_certificate_chain, client_key, client_key_type, client_key_password)
+            self._use_private_key(
+                client_certificate_chain,
+                client_key,
+                client_key_type,
+                client_key_password,
+            )
 
         if ignore_client_authentication_requests:
             if client_certificate_chain:
-                raise ValueError(
-                    "Cannot enable both client_certchain_file and ignore_client_authentication_requests"
-                )
+                raise ValueError("Cannot enable both client_certchain_file and ignore_client_authentication_requests")
 
             self._ssl_ctx.set_client_cert_cb_NULL()
 
@@ -419,6 +429,12 @@ class OpenSslEarlyDataStatusEnum(IntEnum):
     ACCEPTED = 2
 
 
+class ExtendedMasterSecretSupportEnum(IntEnum):
+    NOT_USED_IN_CURRENT_SESSION = 0
+    USED_IN_CURRENT_SESSION = 1
+    UNKNOWN = -1
+
+
 class SslClient(BaseSslClient):
     """High level API implementing an SSL client.
 
@@ -450,6 +466,17 @@ class SslClient(BaseSslClient):
         # TODO(AD): Eventually merge this method with get/set_cipher_list()
         self._ssl.set_ciphersuites(cipher_suites)
 
+    def set_signature_algorithms(self, algorithms: List[Tuple[OpenSslDigestNidEnum, OpenSslEvpPkeyEnum]]) -> None:
+        """Set the enabled signature algorithms for the key exchange.
+
+        The algorithms parameter is a list of a public key algorithm and a digest."""
+        flattened_sigalgs = [item for sublist in algorithms for item in sublist]
+        self._ssl.set1_sigalgs(flattened_sigalgs)
+
+    def get_peer_signature_nid(self) -> OpenSslDigestNidEnum:
+        """Get the digest used for TLS message signing."""
+        return OpenSslDigestNidEnum(self._ssl.get_peer_signature_nid())
+
     def set_groups(self, supported_groups: List[OpenSslEcNidEnum]) -> None:
         """Specify elliptic curves or DH groups that are supported by the client in descending order."""
         self._ssl.set1_groups(supported_groups)
@@ -466,3 +493,15 @@ class SslClient(BaseSslClient):
             raise CertificateChainVerificationFailed(verify_code)
 
         return [x509.as_pem() for x509 in self._ssl.get0_verified_chain()]
+
+    def get_extended_master_secret_support(self) -> ExtendedMasterSecretSupportEnum:
+        """Indicates whether the current session used extended master secret."""
+        support = self._ssl.get_extms_support()
+        if support == 1:
+            return ExtendedMasterSecretSupportEnum.USED_IN_CURRENT_SESSION
+        elif support == 0:
+            return ExtendedMasterSecretSupportEnum.NOT_USED_IN_CURRENT_SESSION
+        elif support == -1:
+            return ExtendedMasterSecretSupportEnum.UNKNOWN
+        else:
+            raise ValueError(f"Unexpected return value get_extms_support(): {support}")
